import json
import datetime
from dateutil.tz import tzutc
import logging

from . import time_parser
from .time_parser import DateTimeWithNS
from . import util


def json2type_value(src_json_str: str, specified_types=None, logger=logging.getLogger("lakeweed__clickhouse")) -> tuple:
    """
    Convert json string to python dict with data types for Clickhouse

    Arguments:
        src_json_str {str} -- JSON string

    Keyword Arguments:
        logger -- logger object (default: {logging.getLogger("lakeweed__clickhouse")})

    Returns:
        tuple -- return tuple (types, values). types={"column":"type on clickhouse", ...}, values={"column":"value on clickhouse", ...}
    """

    if specified_types is None:
        specified_types = {}

    # flatten
    body = json.loads(src_json_str)
    flatten_body = util.flatten(body, delimiter="__")

    # convert types
    casted_body = util.traverse_casting(flatten_body, specified_types)

    # specified type
    types = {}
    values = {}

    for key, value in casted_body.items():
        __json2lcickhouse_sub(key, value, types, values)

    return (types, values)


def __json2clickhouse_sub_list(key, list, types, values):
    items = []
    items_ns = []
    types[key] = "Array(String)"
    for idx_i, v in enumerate(list):
        if isinstance(v, type(list)) or (type(v) is dict):
            items.append(json.dumps(v))
            continue

        idx = str(idx_i)
        temp_type = {}
        temp_value = {}

        __json2lcickhouse_sub(idx, v, temp_type, temp_value)
        items.append(temp_value[idx])

        types[key] = "Array({})".format(temp_type[idx])

        # for DateTime Array
        if (idx + "_ns") in temp_type.keys():
            items_ns.append(temp_value[idx + "_ns"])
            types[key + "_ns"] = "Array({})".format(temp_type[idx + "_ns"])

    values[key] = items

    # for DateTime Array
    if len(items_ns) > 0:
        values[key + "_ns"] = items_ns

    return


def __json2lcickhouse_sub(key, body, types, values):
    if type(body) is list:
        __json2clickhouse_sub_list(key, body, types, values)
        return

    # is atomic type.
    value = body

    if type(value) is float:
        values[key] = float(value)
        types[key] = "Float64"
        return
    if type(value) is int:
        values[key] = int(value)
        types[key] = "Float64"
        return
    if type(value) is bool:
        values[key] = 1 if value else 0
        types[key] = "UInt8"
        return
    if type(value) is DateTimeWithNS:
        (dt, ns) = value.tupple()
        values[key] = dt
        types[key] = "DateTime"
        # Clickhouse can NOT contain ms in DateTime column.
        values[key + "_ns"] = ns
        types[key + "_ns"] = "UInt32"
        return
    if value is None:
        values[key] = None
        types[key] = "String" # Todo: Need to find the table is already created.
        return

    values[key] = str(value)
    types[key] = "String"

    return



from setuptools import setup

setup(
    name="lakeweed",
    version="0.1.1",
    author="TAC",
    author_email="tac@tac42.net",
    description=("Elastic converter JSON to RDB record"),
    license="MIT",
    keywords="json clickhouse",
    url="https://github.com/tac0x2a/lake_weed",
    packages=['lakeweed'],
)




from . import time_parser


def flatten(src: dict, target=None, prefix="", delimiter="."):

    if target == None:
        target = {}

    for k, v in src.items():
        key_name = f"{prefix}{k}"

        if type(v) is dict:
            flatten(v, target=target, prefix=f"{key_name}{delimiter}", delimiter=delimiter)
        else:
            target[key_name] = v

    return target


def traverse_casting(flatten_dict: dict, specified_types=None) -> dict:
    if specified_types == None:
        specified_types = {}

    new_items = {}
    for k, v in flatten_dict.items():

        # Convert specified type
        if k in specified_types.keys():
            specified_type = specified_types[k]

            try:
                new_items[k] = __cast_specified(v, specified_type)
            except TypeError:
                new_items[k] = v
                pass  # TODO logging or abort

        # Convert elastic type
        else:
            new_items[k] = __datetime_parse(v)

    return new_items


def __datetime_parse(value):
    # list
    if type(value) is list:
        return __traverse_datetime_parse_list(value)

    # scala
    if type(value) is str:
        try:
            return time_parser.elastic_time_parse(value)
        except ValueError:
            return value

    return value


def __traverse_datetime_parse_list(list: list):
    try:
        return [time_parser.elastic_time_parse(v) for v in list]
    except (ValueError, TypeError):
        return list


def __cast_specified(original, specified: str):
    s = specified.upper()
    if s in ["STR", "STRING"]:
        return str(original)
    if s in ["INT", "INTEGER"]:
        return int(original)
    if s in ["FLOAT", "DOUBLE", "DECIMAL", "DEC"]:
        return float(original)
    raise TypeError(f"Not supported specified type: {specified}")
